@@ -11,7 +11,6 @@ import Control.Monad.State
 import Data.Maybe
 import System.IO
 import Data.List
-import Data.Tuple
 import qualified Data.Map as M
 
 import qualified Annex
@@ -134,6 +133,7 @@ backend_usage = stat "backend usage" $
 		splits :: [Key] -> [(String, Integer)]
 		splits ks = M.toList $ M.fromListWith (+) $ map tcount ks
 		tcount k = (keyBackendName k, 1)
+		swap (a, b) = (b, a)
 		pp c [] = c
 		pp c ((n, b):xs) = "\n\t" ++ b ++ ": " ++ show n ++ pp c xs
 
