@@ -62,8 +62,8 @@ directorySetup u c = do
 	-- verify configuration is sane
 	let dir = maybe (error "Specify directory=") id $
 		M.lookup "directory" c
-	e <- liftIO $ doesDirectoryExist dir
-	when (not e) $ error $ "Directory does not exist: " ++ dir
+	liftIO $ doesDirectoryExist dir
+		>>! error $ "Directory does not exist: " ++ dir
 	c' <- encryptionSetup c
 
 	-- The directory is stored in git config, not in this remote's
