@@ -7,14 +7,10 @@
 
 module Types (
 	Annex,
-	AnnexState,
 	Backend,
-	Key,
-	genKey,
-	backendName,
-	keyName,
-	FlagName,
-	Flag(..)
+	Key
 ) where
 
-import TypeInternals
+import BackendClass
+import Annex
+import Key
